@@ -33,9 +33,9 @@ from .itertools import (
     zip_longest,
     groupby,
 )
-from .asynctools import borrow, scoped_iter, await_each, apply
+from .asynctools import borrow, scoped_iter, await_each, apply, sync
 
-__version__ = "3.9.2"
+__version__ = "3.10.0"
 
 __all__ = [
     "anext",
@@ -82,4 +82,5 @@ __all__ = [
     "scoped_iter",
     "await_each",
     "apply",
+    "sync",
 ]
