@@ -1,3 +1,5 @@
+from asyncio import iscoroutinefunction
+from functools import wraps
 from typing import (
     Union,
     AsyncIterator,
@@ -9,26 +11,15 @@ from typing import (
     Callable,
     Any,
     overload,
+    Optional,
 )
-import sys
 
-if sys.version_info[:2] >= (3, 8):
-    from typing import AsyncContextManager
-else:
-    from typing_extensions import AsyncContextManager
-
-from ._core import AnyIterable, aiter
+from ._typing import AsyncContextManager, T, T1, T2, T3, T4, T5, AnyIterable
+from ._core import aiter
 from .contextlib import nullcontext
 
 
-T = TypeVar("T")
 S = TypeVar("S")
-# Variadic overloads
-T1 = TypeVar("T1")
-T2 = TypeVar("T2")
-T3 = TypeVar("T3")
-T4 = TypeVar("T4")
-T5 = TypeVar("T5")
 
 
 class _BorrowedAsyncIterator(AsyncGenerator[T, S]):
@@ -36,29 +27,38 @@ class _BorrowedAsyncIterator(AsyncGenerator[T, S]):
     Borrowed async iterator/generator, preventing to ``aclose`` the ``iterable``
     """
 
+    # adding special methods such as `__aiter__` as `__slots__` allows to set them
+    # on the instance: the interpreter expects *descriptors* not methods, and
+    # `__slots__` are descriptors just like methods.
     __slots__ = "__wrapped__", "__aiter__", "__anext__", "asend", "athrow"
 
+    # Type checker does not understand `__slot__` definitions
+    __aiter__: Callable[[Any], AsyncGenerator[T, S]]
+    __anext__: Callable[[Any], Awaitable[T]]
+    asend: Any
+    athrow: Any
+
     def __init__(self, iterator: Union[AsyncIterator[T], AsyncGenerator[T, S]]):
         self.__wrapped__ = iterator
         # iterator.__aiter__ is likely to return iterator (e.g. for async def: yield)
         # We wrap it in a separate async iterator/generator to hide its __aiter__.
         try:
             wrapped_iterator: AsyncGenerator[T, S] = self._wrapped_iterator(iterator)
-            self.__anext__ = iterator.__anext__  # argument must be an async iterable!
+            self.__anext__ = iterator.__anext__  # type: ignore
+            self.__aiter__ = wrapped_iterator.__aiter__  # type: ignore
         except (AttributeError, TypeError):
             raise TypeError(
                 "borrowing requires an async iterator "
                 + f"with __aiter__ and __anext__ method, got {type(iterator).__name__}"
             ) from None
-        self.__aiter__ = wrapped_iterator.__aiter__
-        self.__anext__ = wrapped_iterator.__anext__
+        self.__anext__ = wrapped_iterator.__anext__  # type: ignore
         # Our wrapper cannot pass on asend/athrow without getting much heavier.
         # Since interleaving anext/asend/athrow is not allowed, and the wrapper holds
         # no internal state other than the iterator, circumventing it should be fine.
         if hasattr(iterator, "asend"):
-            self.asend = iterator.asend
+            self.asend = iterator.asend  # type: ignore
         if hasattr(iterator, "athrow"):
-            self.athrow = iterator.athrow
+            self.athrow = iterator.athrow  # type: ignore
 
     # Py3.6 compatibility
     # Use ``(item async for item in iterator)`` inside
@@ -111,7 +111,7 @@ class _ScopedAsyncIteratorContext(AsyncContextManager[AsyncIterator[T]]):
 
     def __init__(self, iterator: AsyncIterator[T]):
         self._iterator: AsyncIterator[T] = iterator
-        self._borrowed_iter = None
+        self._borrowed_iter: Optional[_ScopedAsyncIterator[T, Any]] = None
 
     async def __aenter__(self) -> AsyncIterator[T]:
         if self._borrowed_iter is not None:
@@ -120,8 +120,8 @@ class _ScopedAsyncIteratorContext(AsyncContextManager[AsyncIterator[T]]):
         return borrowed_iter
 
     async def __aexit__(self, exc_type, exc_val, exc_tb):
-        await self._borrowed_iter._aclose_wrapper()
-        await self._iterator.aclose()
+        await self._borrowed_iter._aclose_wrapper()  # type: ignore
+        await self._iterator.aclose()  # type: ignore
         return False
 
     def __repr__(self):
@@ -207,18 +207,18 @@ async def await_each(awaitables: Iterable[Awaitable[T]]) -> AsyncIterable[T]:
 
         import asyncstdlib as a
 
-         async def check1() -> bool:
-              ...
+        async def check1() -> bool:
+            ...
 
         async def check2() -> bool:
-              ...
+            ...
 
         async def check3() -> bool:
-              ...
+            ...
 
-         okay = await a.all(
-             a.await_each(
-                 [check1(), check2(), check3()]))
+        okay = await a.all(
+            a.await_each([check1(), check2(), check3()])
+        )
     """
     for awaitable in awaitables:
         yield await awaitable
@@ -324,3 +324,58 @@ async def apply(
     return __func(
         *[await arg for arg in args], **{k: await arg for k, arg in kwargs.items()}
     )
+
+
+@overload
+def sync(function: Callable[..., Awaitable[T]]) -> Callable[..., Awaitable[T]]:
+    ...
+
+
+@overload
+def sync(function: Callable[..., T]) -> Callable[..., Awaitable[T]]:
+    ...
+
+
+def sync(function: Callable) -> Callable[..., Awaitable[T]]:
+    r"""
+    Wraps a callable to ensure its result can be ``await``\ ed
+
+    Useful to write :term:`async neutral` functions by wrapping callable arguments,
+    or to use synchronous functions where asynchronous ones are expected.
+    Wrapping a regular function defined using ``def`` or ``lambda`` makes it
+    behave roughly as if it were defined using ``async def`` instead.
+
+    Example:
+
+    .. code-block:: python3
+
+        import asyncstdlib as a
+
+        def test1_sync(x, y):
+            ...
+
+        async def test1_async(x):
+            ...
+
+        async def main():
+            await a.sync(test1_sync)(x=1, y=2)
+            await a.sync(test1_async)(x=8)
+            await a.sync(lambda x: x ** 3)(x=5)
+
+        if __name__ == "__main__":
+            asyncio.run(main())
+    """
+    if not callable(function):
+        raise TypeError("function argument should be Callable")
+
+    if iscoroutinefunction(function):
+        return function
+
+    @wraps(function)
+    async def async_wrapped(*args, **kwargs):
+        result = function(*args, **kwargs)
+        if isinstance(result, Awaitable):
+            return await result
+        return result
+
+    return async_wrapped
